@@ -261,7 +261,7 @@ get_changed_events (ECalBackendExchange *cbex, const char *since)
 		modtime = e2k_properties_get_prop (result->props,
 						   E2K_PR_DAV_LAST_MODIFIED);
 
-		if (!e_cal_backend_exchange_in_cache (cbex, uid, modtime)) {
+		if (!e_cal_backend_exchange_in_cache (cbex, uid, modtime, result->href)) {
 			g_ptr_array_add (hrefs, g_strdup (result->href));
 			g_hash_table_insert (modtimes, g_strdup (result->href),
 					     g_strdup (modtime));
@@ -415,6 +415,7 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	const char *summary;
 	E2kHTTPStatus http_status;
 	struct _cb_data *cbdata;
+	GSList *categories;
 	
 	cbexc =	E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
 	
@@ -498,7 +499,6 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	
 	lastmod = e2k_timestamp_from_icaltime (current);
 
-
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, icalcomp);	
 
@@ -507,10 +507,6 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	cbdata->vcal_comp = e_cal_util_new_top_level ();
 	cbdata->cal = cal;
 
-	/*icalprop = icalproperty_new (ICAL_METHOD_PROPERTY);
-	icalproperty_set_method (icalprop, method);
-	icalcomponent_add_property (cbdata->vcal_comp, icalprop);*/
-	
 	/* Remove X parameters from properties */
 	/* This is specifically for X-EVOLUTION-END-DATE, 
 	   but removing anything else is probably ok too */
@@ -543,8 +539,6 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	summary = icalcomponent_get_summary (real_icalcomp);
 	if (!summary)
 		summary = "";
-
-
 	
 	date = e_cal_backend_exchange_make_timestamp_rfc822 (time (NULL));
 	from = e_cal_backend_exchange_get_from_string (backend, comp);
@@ -572,18 +566,13 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	if (http_status != E2K_HTTP_CREATED)
 		return GNOME_Evolution_Calendar_OtherError;
 
-	#if 0
-	ctx = exchange_account_get_context (E_CAL_BACKEND_EXCHANGE (cbexc)->account);	
-	
-	/* PUT the iCal object in the Exchange server */
-	http_status = e2k_context_put (ctx, NULL, location,
-						"message/rfc822", msg, strlen (msg), uid);
-
-	#endif
-	
 	/*add object*/
 	e_cal_backend_exchange_add_object (E_CAL_BACKEND_EXCHANGE (cbexc), location, lastmod, icalcomp);
 	
+	e_cal_component_get_categories_list (comp, &categories);
+	e_cal_backend_ref_categories (E_CAL_BACKEND (cbexc), categories);
+	e_cal_component_free_categories_list (categories);
+	
 	g_free (lastmod);
 	/*cleanup ?*/	
 	
@@ -708,6 +697,7 @@ modify_object (ECalBackendSync *backend, EDataCal *cal,
 	char *from, *date;
 	const char *summary;
 	E2kContext *ctx;
+	GSList *categories;
 
 	cbexc =	E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
 
@@ -774,7 +764,7 @@ modify_object (ECalBackendSync *backend, EDataCal *cal,
 		dt.value->is_date = FALSE;
 		dt.value->is_utc = FALSE;
 		dt.value->hour = dt.value->minute = dt.value->second = 0;
-		dt.value->zone = icaltimezone_get_tzid (zone);
+		dt.value->zone = zone;
 		dt.tzid = icaltimezone_get_tzid (zone);
 		e_cal_component_set_dtstart (tmp_comp, &dt);
 
@@ -782,7 +772,7 @@ modify_object (ECalBackendSync *backend, EDataCal *cal,
 		dt.value->is_date = FALSE;
 		dt.value->is_utc = FALSE;
 		dt.value->hour = dt.value->minute = dt.value->second = 0;
-		dt.value->zone = icaltimezone_get_tzid (zone);
+		dt.value->zone = zone;
 		dt.tzid = icaltimezone_get_tzid (zone);
 		e_cal_component_set_dtend (tmp_comp, &dt);
 	}
@@ -806,7 +796,8 @@ modify_object (ECalBackendSync *backend, EDataCal *cal,
 			else {
 				char *izone;
 				get_timezone (backend, cal, dt.tzid, &izone);
-				from_zone = icalcomponent_new_from_string (izone);
+				from_zone = icalcomponent_get_timezone (icalcomponent_new_from_string (izone),
+											dt.tzid);
 			}
 			to_zone = icaltimezone_get_utc_timezone ();
 
@@ -880,16 +871,26 @@ modify_object (ECalBackendSync *backend, EDataCal *cal,
 	g_free (from);
 	g_free (body_crlf);
 
+	/*unref the old set of categories*/
+	e_cal_component_get_categories_list (old_comp, &categories);
+	e_cal_backend_unref_categories (E_CAL_BACKEND (cbexc), categories);
+	e_cal_component_free_categories_list (categories);
+	
+	/*ref the new set of categories*/
+	e_cal_component_get_categories_list (tmp_comp, &categories);
+	e_cal_backend_ref_categories (E_CAL_BACKEND (cbexc), categories);
+	e_cal_component_free_categories_list (categories);
+
 	ctx = exchange_account_get_context (E_CAL_BACKEND_EXCHANGE (cbexc)->account);	
 	
 	/* PUT the iCal object in the Exchange server */
 	http_status = e2k_context_put (ctx, NULL, ecomp->href, "message/rfc822",
 				       		msg, strlen (msg), NULL);
-					   
+
 	if (E2K_HTTP_STATUS_IS_SUCCESSFUL (http_status))
 		e_cal_backend_exchange_modify_object (E_CAL_BACKEND_EXCHANGE (cbexc), 
 							real_icalcomp, mod);
-
+	
 	g_free (msg);
 	g_object_unref (tmp_comp);
 	icalcomponent_free (cbdata->vcal_comp);
@@ -908,6 +909,10 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 	E2kHTTPStatus status;
 	E2kContext *ctx;
 	ECalComponent *comp;
+	GSList *categories;
+	char *calobj, *obj;
+	struct icaltimetype time_rid;
+	ECalBackendSyncStatus ebs_status;
 	
 	cbexc = E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
 
@@ -922,15 +927,39 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 	e_cal_component_set_icalcomponent (comp, ecomp->comp);
 	*object = e_cal_component_get_as_string (comp);
 	
-	ctx = exchange_account_get_context (E_CAL_BACKEND_EXCHANGE (cbexc)->account);	
-	
-	status = e2k_context_delete (ctx, NULL, ecomp->href);
-	
-	if (E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
-		if (e_cal_backend_exchange_remove_object (E_CAL_BACKEND_EXCHANGE (cbexc), uid))
-			return GNOME_Evolution_Calendar_Success;
+	switch (mod) {
+		
+		case CALOBJ_MOD_ALL:
+			ctx = exchange_account_get_context (E_CAL_BACKEND_EXCHANGE (cbexc)->account);	
+			
+			status = e2k_context_delete (ctx, NULL, ecomp->href);			
+			if (E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {		
+				e_cal_component_get_categories_list (comp, &categories);
+				e_cal_backend_unref_categories (E_CAL_BACKEND (cbexc), categories);
+				e_cal_component_free_categories_list (categories);
+				
+				if (e_cal_backend_exchange_remove_object (E_CAL_BACKEND_EXCHANGE (cbexc), uid))
+						return GNOME_Evolution_Calendar_Success;
+			}
+			break;
+		case CALOBJ_MOD_THIS:
+			/*remove_instance and modify */
+			time_rid = icaltime_from_string (rid);
+			e_cal_util_remove_instances (ecomp->comp, time_rid, mod);
+			
+			calobj  = (char *) icalcomponent_as_ical_string (ecomp->comp);
+			ebs_status = modify_object (backend, cal, calobj, mod, &obj);
+			if (ebs_status != GNOME_Evolution_Calendar_Success)
+				goto error;
+			
+			g_free (obj);
+			return ebs_status;
+		
+		default:
+			break;
 	}
-	
+
+error:
 	return GNOME_Evolution_Calendar_OtherError;
 }
 
@@ -939,8 +968,6 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 		 const char *calobj)
 {
 	ECalBackendExchangeCalendar *cbexc;
-	icalcomponent *toplevel_comp, *icalcomp;
-	icalcomponent_kind kind;
 	ECalComponent *comp;
 	GList *comps, *l;
 	struct icaltimetype current;
@@ -953,39 +980,10 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE_CALENDAR (cbexc), GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_InvalidObject);
 	
-	toplevel_comp = icalparser_parse_string (calobj);
-	if (!toplevel_comp)
+	status = e_cal_backend_exchange_extract_components (calobj, &method, &comps);
+	if (status != GNOME_Evolution_Calendar_Success)
 		return GNOME_Evolution_Calendar_InvalidObject;
-	
-	kind = icalcomponent_isa (toplevel_comp);
-	
-	if (kind != ICAL_VCALENDAR_COMPONENT) {
-		
-		icalcomp = toplevel_comp;
-		toplevel_comp = e_cal_util_new_top_level ();
-		icalcomponent_add_component (toplevel_comp, icalcomp);	
-	}
 
-	method = icalcomponent_get_method (toplevel_comp);
-	
-	comps = NULL;
-	subcomp = icalcomponent_get_first_component (toplevel_comp, ICAL_ANY_COMPONENT);
-	
-	while (subcomp) {
-		
-		icalcomponent_kind child_kind = icalcomponent_isa (subcomp);
-		
-		if (child_kind == ICAL_VEVENT_COMPONENT) {
-			
-			if (!icalcomponent_get_uid (subcomp))
-				return GNOME_Evolution_Calendar_InvalidObject;
-			
-			comps = g_list_prepend (comps, subcomp);
-		}
-		
-		subcomp = icalcomponent_get_next_component (toplevel_comp, ICAL_ANY_COMPONENT);
-	}
-	
 	for (l = comps; l; l= l->next) {
 		const char *uid, *rid;
 		char *calobj;
@@ -1103,7 +1101,7 @@ set_freebusy_info (icalcomponent *vfb, const char *data, time_t start)
 	const char *span_start, *span_end;
 	E2kBusyStatus busy;
 	icalproperty *prop;
-	time_t end;
+	time_t end = 0;
 
 	for (span_start = span_end = data; *span_start; span_start = span_end, start = end) {
 		busy = E2K_FBCHAR_TO_BUSYSTATUS (*span_start);
