@@ -578,7 +578,7 @@ get_changed_tasks (ECalBackendExchange *cbex, const char *since)
 		modtime = e2k_properties_get_prop (result->props,
 						   E2K_PR_DAV_LAST_MODIFIED);
 
-		if (!e_cal_backend_exchange_in_cache (cbex, uid, modtime)) {
+		if (!e_cal_backend_exchange_in_cache (cbex, uid, modtime, result->href)) {
 			g_ptr_array_add (hrefs, g_strdup (result->href));
 			g_hash_table_insert (modtimes, g_strdup (result->href),
 					     g_strdup (modtime));
@@ -866,7 +866,7 @@ create_task_object (ECalBackendSync *backend, EDataCal *cal,
 	modtime = e2k_timestamp_from_icaltime (current);
 
 	/* check if the object is already present in our cache */
-	if (e_cal_backend_exchange_in_cache (E_CAL_BACKEND_EXCHANGE (backend), *comp_uid, modtime)) {
+	if (e_cal_backend_exchange_in_cache (E_CAL_BACKEND_EXCHANGE (backend), *comp_uid, modtime, NULL)) {
 		icalcomponent_free (icalcomp);
 		return GNOME_Evolution_Calendar_ObjectIdAlreadyExists;
 	}	
@@ -934,89 +934,6 @@ create_task_object (ECalBackendSync *backend, EDataCal *cal,
 	return GNOME_Evolution_Calendar_Success;
 }
 
-static const char *
-get_rid_string (ECalComponent *comp)
-{
-        ECalComponentRange range;
-        struct icaltimetype tt;
-        
-	e_cal_component_get_recurid (comp, &range);
-        if (!range.datetime.value)
-                return "0";
-        tt = *range.datetime.value;
-        e_cal_component_free_range (&range);
-        
-	return icaltime_is_valid_time (tt) && !icaltime_is_null_time (tt) ?
-                icaltime_as_ical_string (tt) : "0";
-}
-
-/* g_hash_table_foreach() callback to destroy recurrences in the hash table */
-static void
-free_recurrence (gpointer key, gpointer value, gpointer data)
-{
-        char *rid = key;
-        ECalComponent *comp = value;
-        
-	g_free (rid);
-        g_object_unref (comp);
-}
-
-
-/* g_hash_table_foreach() callback to destroy a ECalBackendExchangeTasksObject */
-static void
-free_object (gpointer key, gpointer value, gpointer data)
-{
-        ECalBackendExchangeTasksObject *obj_data = value;
-        
-	g_object_unref (obj_data->full_object);
-        g_hash_table_foreach (obj_data->recurrences, (GHFunc) free_recurrence, NULL);
-        g_hash_table_destroy (obj_data->recurrences);
-}
-
-/* g_hash_table_foreach() callback to remove recurrences from the calendar */
-static void
-remove_recurrence_cb (gpointer key, gpointer value, gpointer data)
-{
-        GList *l;
-        GSList *categories;
-        icalcomponent *icalcomp;
-        ECalBackendExchangeTasksPrivate *priv;
-        ECalComponent *comp = value;
-        ECalBackendExchangeTasks *cbext = data;
-        
-	priv = cbext->priv;
-        
-	/* remove the recurrence from the top-level calendar */
-        icalcomp = e_cal_component_get_icalcomponent (comp);
-        g_assert (icalcomp != NULL);
-        
-	icalcomponent_remove_component (priv->icalcomp, icalcomp);
-        
-	/* remove it from our mapping */
-        l = g_list_find (priv->comp, comp);
-        priv->comp = g_list_delete_link (priv->comp, l);
-        
-	/* update the set of categories */
-        e_cal_component_get_categories_list (comp, &categories);
-        e_cal_backend_unref_categories (E_CAL_BACKEND (cbext), categories);
-        e_cal_component_free_categories_list (categories);
-}
-
-static struct icaltimetype
-get_rid_icaltime (ECalComponent *comp)
-{
-        ECalComponentRange range;
-        struct icaltimetype tt;
-        
-	e_cal_component_get_recurid (comp, &range);
-        if (!range.datetime.value)
-                return icaltime_null_time ();
-        tt = *range.datetime.value;
-        e_cal_component_free_range (&range);
-        
-	return tt;
-}
-
 static ECalBackendSyncStatus
 modify_task_object (ECalBackendSync *backend, EDataCal *cal,
 	       const char *calobj, CalObjModType mod,
@@ -1106,12 +1023,70 @@ modify_task_object (ECalBackendSync *backend, EDataCal *cal,
 }
 
 static ECalBackendSyncStatus
-receive_objects (ECalBackendSync *backend, EDataCal *cal,
+receive_task_objects (ECalBackendSync *backend, EDataCal *cal,
                  const char *calobj)
 {
-        /* FIXME */
-	return e_cal_backend_exchange_receive_objects (E_CAL_BACKEND_EXCHANGE(backend), cal, calobj);
-        /* return GNOME_Evolution_Calendar_OtherError; */
+	ECalBackendExchangeTasks *ecalbextask;
+        ECalComponent *ecalcomp;
+        GList *comps, *l;
+        struct icaltimetype current;
+        icalproperty_method method;
+        icalcomponent *subcomp;
+        ECalBackendSyncStatus status = GNOME_Evolution_Calendar_Success;
+                                                                                
+	ecalbextask = E_CAL_BACKEND_EXCHANGE_TASKS (backend);
+	
+	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE_TASKS (ecalbextask),
+                                        GNOME_Evolution_Calendar_NoSuchCal);
+        g_return_val_if_fail (calobj != NULL,
+                                GNOME_Evolution_Calendar_ObjectNotFound);
+                                                                                
+	status = e_cal_backend_exchange_extract_components (calobj, &method, &comps);
+        if (status != GNOME_Evolution_Calendar_Success)
+                return GNOME_Evolution_Calendar_InvalidObject;
+
+	for (l = comps; l; l = l->next) {
+                const char *uid, *rid;
+                char *calobj;
+                                                                                
+                subcomp = l->data;
+                                                                                
+                ecalcomp = e_cal_component_new ();
+                e_cal_component_set_icalcomponent (ecalcomp, subcomp);
+                                                                                
+                current = icaltime_from_timet (time (NULL), 0);
+                e_cal_component_set_created (ecalcomp, &current);
+                e_cal_component_set_last_modified (ecalcomp, &current);
+                                                                                
+                /*sanitize?*/
+
+		e_cal_component_get_uid (ecalcomp, &uid);
+                rid = e_cal_component_get_recurid_as_string (ecalcomp);
+                                                                                
+                /*see if the object is there in the cache. if found, modify object, else create object*/
+                                                                                
+                if (get_exchange_comp (E_CAL_BACKEND_EXCHANGE (ecalbextask), uid)) {
+                        char *old_object;
+                        status = modify_task_object (backend, cal, calobj, CALOBJ_MOD_THIS, &old_object);
+                        if (status != GNOME_Evolution_Calendar_Success)
+                                goto error;
+                                                                                
+                        e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend), old_object, calobj);
+			g_free (old_object);
+                } else {
+                        char *returned_uid;
+			calobj = (char *) icalcomponent_as_ical_string (subcomp);
+			status = create_task_object (backend, cal, &calobj, &returned_uid);
+                        if (status != GNOME_Evolution_Calendar_Success)
+                                goto error;
+                                                                                
+                        e_cal_backend_notify_object_created (E_CAL_BACKEND (backend), calobj);
+                }
+        }
+                                                                                
+        g_list_free (comps);
+error:
+        return status;
 }
 
 static ECalBackendSyncStatus
@@ -1181,7 +1156,7 @@ class_init (ECalBackendExchangeTasksClass *klass)
 	sync_class->create_object_sync = create_task_object;
 	sync_class->modify_object_sync = modify_task_object;
 	sync_class->remove_object_sync = remove_task_object;
-	sync_class->receive_objects_sync = receive_objects;
+	sync_class->receive_objects_sync = receive_task_objects;
 
 	object_class->dispose = dispose;
 	object_class->finalize = finalize;
