@@ -57,7 +57,7 @@ struct _ExchangeAccountPrivate {
 	GHashTable *standard_uris;
 
 	GMutex *connect_lock;
-	gboolean connected;
+	gboolean connecting, connected;
 
 	GPtrArray *hierarchies;
 	GHashTable *hierarchies_by_folder, *foreign_hierarchies;
@@ -82,7 +82,6 @@ enum {
 	CONNECTED,
 	NEW_FOLDER,
 	REMOVED_FOLDER,
-	UPDATED_FOLDER,
 	LAST_SIGNAL
 };
 
@@ -132,15 +131,6 @@ class_init (GObjectClass *object_class)
 			      e2k_marshal_NONE__POINTER,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_POINTER);
-	signals[UPDATED_FOLDER] =
-		g_signal_new ("updated_folder",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (ExchangeAccountClass, updated_folder),
-			      NULL, NULL,
-			      e2k_marshal_NONE__POINTER,
-			      G_TYPE_NONE, 1,
-			      G_TYPE_POINTER);
 }
 
 static void
@@ -368,14 +358,6 @@ hierarchy_removed_folder (ExchangeHierarchy *hier, EFolder *folder,
 	g_object_unref (folder);
 }
 
-static void
-hierarchy_updated_folder (ExchangeHierarchy *hier, EFolder *folder,
-			  ExchangeAccount *account)
-{
-	g_signal_emit (account, signals[UPDATED_FOLDER], 0, folder);
-}
-
-
 static gboolean
 get_folder (ExchangeAccount *account, const char *path,
 	    EFolder **folder, ExchangeHierarchy **hier)
@@ -474,21 +456,6 @@ exchange_account_xfer_folder (ExchangeAccount *account,
 					       remove_source);
 }
 
-/**
- * exchange_account_update_folder:
- * @account: the account
- * @folder: the folder to update
- *
- * Tells the shell to update the unread count on the indicated folder.
- **/
-void
-exchange_account_update_folder (ExchangeAccount *account, EFolder *folder)
-{
-	g_return_if_fail (EXCHANGE_IS_ACCOUNT (account));
-
-	g_signal_emit (account, signals[UPDATED_FOLDER], 0, folder);
-}
-
 static void
 remove_hierarchy (ExchangeAccount *account, ExchangeHierarchy *hier)
 {
@@ -503,7 +470,6 @@ remove_hierarchy (ExchangeAccount *account, ExchangeHierarchy *hier)
 	g_hash_table_remove (account->priv->foreign_hierarchies,
 			     hier->owner_email);
 	g_signal_handlers_disconnect_by_func (hier, hierarchy_new_folder, account);
-	g_signal_handlers_disconnect_by_func (hier, hierarchy_updated_folder, account);
 	g_signal_handlers_disconnect_by_func (hier, hierarchy_removed_folder, account);
 	g_object_unref (hier);
 }
@@ -515,8 +481,6 @@ setup_hierarchy (ExchangeAccount *account, ExchangeHierarchy *hier)
 
 	g_signal_connect (hier, "new_folder",
 			  G_CALLBACK (hierarchy_new_folder), account);
-	g_signal_connect (hier, "updated_folder",
-			  G_CALLBACK (hierarchy_updated_folder), account);
 	g_signal_connect (hier, "removed_folder",
 			  G_CALLBACK (hierarchy_removed_folder), account);
 
@@ -854,7 +818,7 @@ get_password (ExchangeAccount *account, E2kAutoconfig *ac, const char *errmsg)
 			NULL);
 		if (remember != oldremember) {
 			account->priv->account->source->save_passwd = remember;
-			e_account_list_save (account->priv->account_list);			}
+		}
 		g_free (prompt);
 	}
 	g_free (key);
@@ -971,11 +935,17 @@ exchange_account_connect (ExchangeAccount *account)
 
 	g_mutex_lock (account->priv->connect_lock);
 
-	if (account->priv->ctx) {
+	if (account->priv->connecting) {
+		g_mutex_unlock (account->priv->connect_lock);
+		return NULL;
+	} else if (account->priv->ctx) {
 		g_mutex_unlock (account->priv->connect_lock);
 		return account->priv->ctx;
 	}
 
+	account->priv->connecting = TRUE;
+	g_mutex_unlock (account->priv->connect_lock);
+
 	ac = e2k_autoconfig_new (account->home_uri,
 				 account->priv->username,
 				 NULL,
@@ -985,7 +955,7 @@ exchange_account_connect (ExchangeAccount *account)
 
  try_password_again:
 	if (!get_password (account, ac, errmsg)) {
-		g_mutex_unlock (account->priv->connect_lock);
+		account->priv->connecting = FALSE;
 		return NULL;
 	}
 
@@ -1040,7 +1010,7 @@ exchange_account_connect (ExchangeAccount *account)
 		}
 
 		e2k_autoconfig_free (ac);
-		g_mutex_unlock (account->priv->connect_lock);
+		account->priv->connecting = FALSE;
 
 		if (!exchange_component_is_interactive (global_exchange_component))
 			return NULL;
@@ -1110,7 +1080,7 @@ exchange_account_connect (ExchangeAccount *account)
 				       &results, &nresults);
 
 	if (!E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
-		g_mutex_unlock (account->priv->connect_lock);
+		account->priv->connecting = FALSE;
 		return NULL;
 	}
 
@@ -1211,7 +1181,7 @@ exchange_account_connect (ExchangeAccount *account)
 	fresult = exchange_hierarchy_scan_subtree (personal_hier,
 						   personal_hier->toplevel);
 	if (fresult != EXCHANGE_ACCOUNT_FOLDER_OK) {
-		g_mutex_unlock (account->priv->connect_lock);
+		account->priv->connecting = FALSE;
 		return NULL;
 	}
 
@@ -1219,18 +1189,16 @@ exchange_account_connect (ExchangeAccount *account)
 		account->priv->favorites_hierarchy,
 		account->priv->favorites_hierarchy->toplevel);
 	if (fresult != EXCHANGE_ACCOUNT_FOLDER_OK) {
-		g_mutex_unlock (account->priv->connect_lock);
+		account->priv->connecting = FALSE;
 		return NULL;
 	}
 	
-
 	account->priv->connected = TRUE;
+	account->priv->connecting = FALSE;
 
 	g_signal_connect (account->priv->ctx, "redirect",
 			  G_CALLBACK (context_redirect), account);
 
-	g_mutex_unlock (account->priv->connect_lock);
-
 	g_signal_emit (account, signals[CONNECTED], 0, account->priv->ctx);
 	return account->priv->ctx;
 }
@@ -1488,7 +1456,7 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 	account->priv->identity_email = g_strdup (adata->id->address);
 
 	/* URI, etc, info */
-	enc_user = e2k_uri_encode (uri->user, "@/;:");
+	enc_user = e2k_uri_encode (uri->user, FALSE, "@/;:");
 	account->priv->uri_authority = g_strdup_printf ("%s@%s", enc_user,
 							uri->host);
 	g_free (enc_user);
@@ -1540,7 +1508,7 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 		param = uri->user;
 	else if (!g_ascii_strncasecmp (param, account->priv->identity_email, strlen (param)))
 		account->priv->uris_use_email = TRUE;
-	mailbox = e2k_uri_encode (param, "/");
+	mailbox = e2k_uri_encode (param, TRUE, "/");
 	account->home_uri = g_strdup_printf (account->priv->http_uri_schema,
 					     uri->host, mailbox);
 	g_free (mailbox);
@@ -1548,6 +1516,12 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 	param = e2k_uri_get_param (uri, "filter");
 	if (param)
 		account->filter_inbox = TRUE;
+	param = e2k_uri_get_param (uri, "filter_junk");
+	if (param)
+		account->filter_junk = TRUE;
+	param = e2k_uri_get_param (uri, "filter_junk_inbox");
+	if (param)
+		account->filter_junk_inbox_only = TRUE;
 
 	e2k_uri_free (uri);
 
